"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TNSRTCSessionDescription_1 = require("./TNSRTCSessionDescription");
var _1 = require("./");
var TNSRTCPeerConnectionState_1 = require("../core/TNSRTCPeerConnectionState");
var utils = require("tns-core-modules/utils/utils");
var TNSRTCPeerConnection = (function () {
    function TNSRTCPeerConnection(config) {
        this._remoteDescription = null;
        this._localDescription = null;
        var configuration = config && config instanceof _1.TNSRTCConfiguration ? config : new _1.TNSRTCConfiguration();
        this.android = new co.fitcom.fancywebrtc.FancyRTCPeerConnection(utils.ad.getApplicationContext(), configuration.instance);
    }
    Object.defineProperty(TNSRTCPeerConnection.prototype, "localDescription", {
        get: function () {
            return this._localDescription;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCPeerConnection.prototype, "remoteDescription", {
        get: function () {
            return this._remoteDescription;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCPeerConnection.prototype, "connectionState", {
        get: function () {
            switch (this.android.getConnectionState()) {
                case co.fitcom.fancywebrtc.FancyRTCPeerConnectionState.NEW:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.NEW;
                case co.fitcom.fancywebrtc.FancyRTCPeerConnectionState.CONNECTING:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.CONNECTING;
                case co.fitcom.fancywebrtc.FancyRTCPeerConnectionState.CONNECTED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.CONNECTED;
                case co.fitcom.fancywebrtc.FancyRTCPeerConnectionState.DISCONNECTED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.DISCONNECTED;
                case co.fitcom.fancywebrtc.FancyRTCPeerConnectionState.CLOSED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.CLOSED;
                case co.fitcom.fancywebrtc.FancyRTCPeerConnectionState.FAILED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.FAILED;
                default:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.NEW;
            }
        },
        enumerable: true,
        configurable: true
    });
    TNSRTCPeerConnection.prototype.onConnectionStateChange = function (callback) {
        this.android.setOnConnectionStateChange(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnConnectionStateChangeListener({
            onChange: function () {
                callback();
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onTrack = function (callback) {
        this.android.setOnTrackListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnTrackListener({
            onTrack: function (trackEvent) {
                callback(_1.TNSRTCTrackEvent.fromNative(trackEvent));
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onRemoveTrackListener = function (callback) {
        this.android.setOnRemoveTrackListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnRemoveTrackListener({
            onRemoveTrack: function () {
                callback();
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onRemoveStream = function (callback) {
        this.android.setOnRemoveStreamListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnRemoveStreamListener({
            onRemoveStream: function (nativeStream) {
                callback(_1.TNSRTCMediaStream.fromNative(nativeStream));
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onIceGatheringStateChange = function (callback) {
        this.android.setOnIceGatheringStateChangeListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnIceGatheringStateChangeListener({
            onIceGatheringStateChange: function () {
                callback();
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onAddStream = function (callback) {
        this.android.setOnAddStreamListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnAddStreamListener({
            onAddStream: function (nativeStream) {
                callback(_1.TNSRTCMediaStream.fromNative(nativeStream));
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onNegotiationNeeded = function (callback) {
        this.android.setOnNegotiationNeededListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnNegotiationNeededListener({
            onNegotiationNeeded: function () {
                callback();
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onSignalingStateChange = function (callback) {
        this.android.setOnSignalingStateChangeListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnSignalingStateChangeListener({
            onSignalingStateChange: function () {
                callback();
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onIceCandidate = function (callback) {
        this.android.setOnIceCandidateListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnIceCandidateListener({
            onIceCandidate: function (nativeCandidate) {
                callback(_1.TNSRTCIceCandidate.fromNative(nativeCandidate));
            }
        }));
    };
    TNSRTCPeerConnection.prototype.onDataChannel = function (callback) {
        this.android.setOnDataChannelListener(new co.fitcom.fancywebrtc.FancyRTCPeerConnection.FancyOnDataChannelListener({
            onDataChannel: function (event) {
                callback(_1.TNSRTCDataChannelEvent.fromNative(event));
            }
        }));
    };
    TNSRTCPeerConnection.prototype.defaultIceServers = function () {
        var nativeServers = this.android.getDefaultIceServers();
        var size = nativeServers.size();
        var servers = [];
        for (var i = 0; i < size; i++) {
            var nativeServer = nativeServers.get(i);
            servers.push(new _1.TNSRTCIceServer(nativeServer.getUrls(), nativeServer.getUsername(), nativeServer.getCredential()));
        }
        return servers;
    };
    TNSRTCPeerConnection.prototype.addIceCandidate = function (candidate) {
        this.android.addIceCandidate(new co.fitcom.fancywebrtc.FancyRTCIceCandidate(candidate.sdp, candidate.sdpMid, candidate.sdpMLineIndex));
    };
    TNSRTCPeerConnection.prototype.addTrack = function (track, streamIds) {
        if (streamIds) {
            var ids = new java.util.ArrayList();
            for (var _i = 0, streamIds_1 = streamIds; _i < streamIds_1.length; _i++) {
                var id = streamIds_1[_i];
                ids.add(id);
            }
            this.android.addTrack(track.mediaStreamTrack, ids);
        }
    };
    TNSRTCPeerConnection.prototype.close = function () {
        this.android.close();
    };
    TNSRTCPeerConnection.prototype.createDataChannel = function (label, channelInit) {
        var init = new co.fitcom.fancywebrtc.FancyRTCDataChannelInit();
        if (channelInit.id != null) {
            init.getInit().id = channelInit.id;
        }
        if (channelInit.maxPacketLifeTime != null) {
            init.getInit().maxRetransmitTimeMs = channelInit.maxPacketLifeTime;
        }
        if (channelInit.maxRetransmits != null) {
            init.getInit().maxRetransmits = channelInit.maxRetransmits;
        }
        if (channelInit.negotiate != null) {
            init.getInit().negotiated = channelInit.negotiate;
        }
        if (channelInit.protocol != null) {
            init.getInit().protocol = channelInit.protocol;
        }
        if (channelInit.ordered != null) {
            init.getInit().ordered = channelInit.ordered;
        }
        return _1.TNSRTCDataChannel.fromNative(this.android.createDataChannel(label, init));
    };
    TNSRTCPeerConnection.prototype.createAnswer = function (mediaConstraints) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var constraints = new co.fitcom.fancywebrtc.FancyRTCMediaConstraints();
            if (mediaConstraints.mandatory) {
                for (var _i = 0, _a = mediaConstraints.mandatory; _i < _a.length; _i++) {
                    var constraint = _a[_i];
                    var keyValue = new co.fitcom.fancywebrtc.FancyRTCMediaConstraints.KeyValue(constraint.key, constraint.value);
                    constraints.mandatory.add(keyValue);
                }
            }
            if (mediaConstraints.optional) {
                for (var _b = 0, _c = mediaConstraints.optional; _b < _c.length; _b++) {
                    var constraint = _c[_b];
                    var keyValue = new co.fitcom.fancywebrtc.FancyRTCMediaConstraints.KeyValue(constraint.key, constraint.value);
                    constraints.mandatory.add(keyValue);
                }
            }
            _this.android.createAnswer(constraints, new co.fitcom.fancywebrtc.FancyRTCPeerConnection.SdpCreateListener({
                onError: function (error) {
                    reject(error);
                },
                onSuccess: function (sdp) {
                    resolve(TNSRTCSessionDescription_1.TNSRTCSessionDescription.fromNative(sdp));
                }
            }));
        });
    };
    TNSRTCPeerConnection.prototype.createOffer = function (mediaConstraints) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var constraints = new co.fitcom.fancywebrtc.FancyRTCMediaConstraints();
            if (mediaConstraints.mandatory) {
                for (var _i = 0, _a = mediaConstraints.mandatory; _i < _a.length; _i++) {
                    var constraint = _a[_i];
                    var keyValue = new co.fitcom.fancywebrtc.FancyRTCMediaConstraints.KeyValue(constraint.key, constraint.value);
                    constraints.mandatory.add(keyValue);
                }
            }
            if (mediaConstraints.optional) {
                for (var _b = 0, _c = mediaConstraints.optional; _b < _c.length; _b++) {
                    var constraint = _c[_b];
                    var keyValue = new co.fitcom.fancywebrtc.FancyRTCMediaConstraints.KeyValue(constraint.key, constraint.value);
                    constraints.mandatory.add(keyValue);
                }
            }
            _this.android.createOffer(constraints, new co.fitcom.fancywebrtc.FancyRTCPeerConnection.SdpCreateListener({
                onError: function (error) {
                    reject(error);
                },
                onSuccess: function (sdp) {
                    resolve(TNSRTCSessionDescription_1.TNSRTCSessionDescription.fromNative(sdp));
                }
            }));
        });
    };
    TNSRTCPeerConnection.prototype.setLocalDescription = function (sdp) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var ref = new WeakRef(_this);
            _this.android.setLocalDescription(sdp.instance, new co.fitcom.fancywebrtc.FancyRTCPeerConnection.SdpSetListener({
                onSuccess: function () {
                    var owner = ref.get();
                    if (owner) {
                        owner._localDescription = sdp;
                    }
                    resolve();
                }, onError: function (error) {
                    reject(error);
                }
            }));
        });
    };
    TNSRTCPeerConnection.prototype.setRemoteDescription = function (sdp) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var ref = new WeakRef(_this);
            _this.android.setRemoteDescription(sdp.instance, new co.fitcom.fancywebrtc.FancyRTCPeerConnection.SdpSetListener({
                onSuccess: function () {
                    var owner = ref.get();
                    if (owner) {
                        owner._remoteDescription = sdp;
                    }
                    resolve();
                }, onError: function (error) {
                    reject(error);
                }
            }));
        });
    };
    TNSRTCPeerConnection.prototype.dispose = function () {
        this.android.dispose();
    };
    TNSRTCPeerConnection.prototype.getSenders = function () {
        var senders = [];
        var nativeSenders = this.android.getSenders();
        if (nativeSenders) {
            var count = nativeSenders.size();
            for (var i = 0; i < count; i++) {
                senders.push(_1.TNSRTCRtpSender.fromNative(nativeSenders.get(i)));
            }
        }
        return senders;
    };
    return TNSRTCPeerConnection;
}());
exports.TNSRTCPeerConnection = TNSRTCPeerConnection;
//# sourceMappingURL=TNSRTCPeerConnection.js.map